@@ -1,5 +1,5 @@
 /*
-    $Id: cd-info.c,v 1.2 2003/04/26 14:24:44 rocky Exp $
+    $Id: cd-info.c,v 1.14 2003/06/22 22:41:29 rocky Exp $
 
     Copyright (C) 2003 Rocky Bernstein <rocky@panix.com>
     Copyright (C) 1996,1997,1998  Gerd Knorr <kraxel@bytesex.org>
@@ -23,11 +23,8 @@
   CD Info - prints various information about a CD, and detects the type of 
   the CD.
  
-  usage: cdinfo  [options] [ dev ]
- 
 */
 #define PROGRAM_NAME "CD Info"
-#define CDINFO_VERSION "2.0"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -57,7 +54,6 @@
 #include <cdio/util.h>
 
 #include <fcntl.h>
-#include <sys/ioctl.h>
 #ifdef __linux__
 # include <linux/version.h>
 # include <linux/cdrom.h>
@@ -66,7 +62,6 @@
 # endif
 #endif
  
-#include <sys/ioctl.h>
 #include <errno.h>
 
 #ifdef ENABLE_NLS
@@ -100,8 +95,9 @@
 #endif
 
 #define err_exit(fmt, args...) \
-  fprintf(stderr, "%s: "fmt, program_name, ##args); \
-  myexit(EXIT_FAILURE)		     
+  { fprintf(stderr, "%s: "fmt, program_name, ##args);	\
+    myexit(EXIT_FAILURE);				\
+  }							
   
 /*
 Subject:   -65- How can I read an IRIX (EFS) CD-ROM on a machine which
@@ -228,7 +224,7 @@ int                        num_audio = 0;              /* # of audio tracks */
 char *source_name = NULL;
 char *program_name;
 
-const char *argp_program_version     = PROGRAM_NAME CDINFO_VERSION;
+const char *argp_program_version     = PROGRAM_NAME " " VERSION;
 const char *argp_program_bug_address = "rocky@panix.com";
 
 typedef enum
@@ -245,19 +241,26 @@ typedef enum
  */
 struct arguments
 {
-  int no_tracks;
-  int no_ioctl;
-  int no_analysis;
+  int            no_tracks;
+  int            no_ioctl;
+  int            no_analysis;
 #ifdef HAVE_CDDB
-  int no_cddb;
-  int cddb_port;  /* port number to contact CDDB server. */
-  int cddb_http;  /* 1 if use http proxy */
+  int            no_cddb;     /* If set the below are meaningless. */
+  char          *cddb_email;  /* email to report to CDDB server. */
+  char          *cddb_server; /* CDDB server to contact */
+  int            cddb_port;   /* port number to contact CDDB server. */
+  int            cddb_http;   /* 1 if use http proxy */
+  int            cddb_timeout;
+  bool           cddb_disable_cache; /* If set the below is meaningless. */
+  char          *cddb_cachedir;
 #endif
 #ifdef HAVE_VCDINFO
-  int no_vcd;
+  int            no_vcd;
 #endif
-  int debug_level;
-  int silent;
+  uint32_t       debug_level;
+  int            silent;
+  int            version_only;
+  int            no_header;
   source_image_t source_image;
 } opts;
      
@@ -287,20 +290,20 @@ enum {
 char *temp_str;
 
 struct poptOption optionsTable[] = {
-  {"debug",    'd', POPT_ARG_INT, &opts.debug_level, 0,
+  {"debug",       'd', POPT_ARG_INT, &opts.debug_level, 0,
    "Set debugging to LEVEL"},
   
-  {"quiet",    'q', POPT_ARG_NONE, &opts.silent, 0,
+  {"quiet",       'q', POPT_ARG_NONE, &opts.silent, 0,
    "Don't produce warning output" },
   
-  {"no-tracks", 'T', POPT_ARG_NONE, &opts.no_tracks, 0,
+  {"no-tracks",   'T', POPT_ARG_NONE, &opts.no_tracks, 0,
    "Don't show track information"},
   
-  {"no-analyze",'A', POPT_ARG_NONE, &opts.no_analysis, 0,
+  {"no-analyze",  'A', POPT_ARG_NONE, &opts.no_analysis, 0,
    "Don't filesystem analysis"},
   
 #ifdef HAVE_CDDB
-  {"no-cddb",   'a', POPT_ARG_NONE, &opts.no_cddb, 0,
+  {"no-cddb",     'a', POPT_ARG_NONE, &opts.no_cddb, 0,
    "Don't look up audio CDDB information or print that"},
 
   {"cddb-port",   'P', POPT_ARG_INT, &opts.cddb_port, 0,
@@ -308,6 +311,22 @@ struct poptOption optionsTable[] = {
 
   {"cddb-http",   'H', POPT_ARG_NONE, &opts.cddb_http, 0,
    "Lookup CDDB via HTTP proxy (default no proxy)"},
+
+  {"cddb-server", '\0', POPT_ARG_STRING, &opts.cddb_server, 0,
+   "CDDB server to contact for information (default: freedb.freedb.org)"},
+
+  {"cddb-cache",  '\0', POPT_ARG_STRING, &opts.cddb_cachedir, 0,
+   "Location of CDDB cache directory (default ~/.cddbclient)"},
+
+  {"cddb-email",  '\0', POPT_ARG_STRING, &opts.cddb_email, 0,
+   "Email address to give CDDB server (default me@home"},
+
+  {"no-cddb-cache", '\0', POPT_ARG_NONE, &opts.cddb_disable_cache, 0,
+   "Lookup CDDB via HTTP proxy (default no proxy)"},
+
+  {"cddb-timeout",  '\0', POPT_ARG_INT, &opts.cddb_timeout, 0,
+   "CDDB timeout value in seconds (default 10 seconds)"},
+
 #endif
   
 #ifdef HAVE_VCDINFO
@@ -331,13 +350,16 @@ struct poptOption optionsTable[] = {
    OP_SOURCE_DEVICE,
    "set CD-ROM device as source", "DEVICE"},
 
+  {"no-header", '\0', POPT_ARG_NONE, &opts.no_header, 
+   0, "Don't display header and copyright (for regression testing)"},
+  
   {"nrg-file", 'N', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name, 
    OP_SOURCE_NRG, "set Nero CD-ROM disk image file as source", "FILE"},
   
   {"quiet", 'q', POPT_ARG_NONE, &opts.silent, 0, 
    "show only critical messages"},
   
-  {"version", 'V', POPT_ARG_NONE, NULL, OP_VERSION,
+  {"version", 'V', POPT_ARG_NONE, &opts.version_only, 0,
    "display version and copyright information and exit"},
   POPT_AUTOHELP {NULL, 0, 0, NULL, 0}
 };
@@ -346,6 +368,9 @@ struct poptOption optionsTable[] = {
 static char *
 fillout_device_name(const char *device_name) 
 {
+#if defined(HAVE_WIN32_CDROM)
+  return strdup(device_name);
+#else
   unsigned int prefix_len=strlen(DEV_PREFIX);
   if (0 == strncmp(device_name, DEV_PREFIX, prefix_len))
     return strdup(device_name);
@@ -354,6 +379,7 @@ fillout_device_name(const char *device_name)
     sprintf(full_device_name, DEV_PREFIX "%s", device_name);
     return full_device_name;
   }
+#endif
 }
 
 
@@ -428,16 +454,36 @@ parse_options (poptContext optCon)
   
   return true;
 }
-     
+
 static void
-print_version (void)
+print_version (bool version_only)
 {
-  printf( _("CD Info %s | (c) 2003 Gerd Knorr, Heiko Eifeldt & R. Bernstein\n\
-This is free software; see the source for copying conditions.\n\
+  
+  driver_id_t driver_id;
+
+  if (!opts.no_header)
+    printf( _("CD Info %s (c) 2003 Gerd Knorr, Heiko Eifeldt & R. Bernstein\n"),
+	    VERSION);
+  printf( _("This is free software; see the source for copying conditions.\n\
 There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A\n\
 PARTICULAR PURPOSE.\n\
-"),
-	 CDINFO_VERSION);
+"));
+
+  if (version_only) {
+    char *default_device;
+    for (driver_id=DRIVER_UNKNOWN+1; driver_id<=CDIO_MAX_DRIVER; driver_id++) {
+      if (cdio_have_driver(driver_id)) {
+	printf("Have driver: %s\n", cdio_driver_describe(driver_id));
+      }
+    }
+    default_device=cdio_get_default_device(NULL);
+    if (default_device)
+      printf("Default CD-ROM device: %s\n", default_device);
+    else
+      printf("No CD-ROM device found.\n");
+    exit(100);
+  }
+  
 }
 
 /* ------------------------------------------------------------------------ */
@@ -457,12 +503,15 @@ read_block(int superblock, uint32_t offset, uint8_t bufnum, track_t track_num)
   
   dbg_print(2, "about to read sector %u\n", offset+superblock);
 
-  if (0 > cdio_lseek(cdio, CDIO_CD_FRAMESIZE*(offset+superblock),SEEK_SET))
-    return -1;
-  
-
-  if (0 > cdio_read(cdio, buffer[bufnum], CDIO_CD_FRAMESIZE))
-    return -1;
+  if (cdio_get_track_green(cdio,  track_num)) {
+    if (0 > cdio_read_mode2_sector(cdio, buffer[bufnum], 
+				   offset+superblock, false))
+      return -1;
+  } else {
+    if (0 > cdio_read_yellow_sector(cdio, buffer[bufnum], 
+				    offset+superblock, false))
+      return -1;
+  }
 
   return 0;
 }
@@ -470,15 +519,11 @@ read_block(int superblock, uint32_t offset, uint8_t bufnum, track_t track_num)
 static bool 
 is_it(int num) 
 {
-  signature_t *sigp;
-  int len;
+  signature_t *sigp=&sigs[num];
+  int len=strlen(sigp->sig_str);
 
   /* TODO: check that num < largest sig. */
-  sigp = &sigs[num];
-
-  len = strlen(sigp->sig_str);
-  return 0 == memcmp(&buffer[sigp->buf_num][sigp->offset], 
-		     sigp->sig_str, len);
+  return 0 == memcmp(&buffer[sigp->buf_num][sigp->offset], sigp->sig_str, len);
 }
 
 static int 
@@ -716,7 +761,10 @@ cddb_discid()
 
 /* CDIO logging routines */
 
-static cdio_log_handler_t gl_default_log_handler = NULL;
+static cdio_log_handler_t gl_default_cdio_log_handler = NULL;
+#ifdef HAVE_CDDB
+static cddb_log_handler_t gl_default_cddb_log_handler = NULL;
+#endif
 
 static void 
 _log_handler (cdio_log_level_t level, const char message[])
@@ -730,7 +778,7 @@ _log_handler (cdio_log_level_t level, const char message[])
   if (level == CDIO_LOG_WARN  && opts.silent)
     return;
   
-  gl_default_log_handler (level, message);
+  gl_default_cdio_log_handler (level, message);
 }
 
 
@@ -747,13 +795,31 @@ print_cddb_info() {
     goto cddb_destroy;
   }
 
-  cddb_set_email_address(conn, "me@home");
-  cddb_set_server_name(conn, "freedb.freedb.org");
+  if (NULL == opts.cddb_email) 
+    cddb_set_email_address(conn, "me@home");
+  else 
+    cddb_set_email_address(conn, opts.cddb_email);
+
+  if (NULL == opts.cddb_server) 
+    cddb_set_server_name(conn, "freedb.freedb.org");
+  else 
+    cddb_set_server_name(conn, opts.cddb_server);
+
+  if (opts.cddb_timeout >= 0) 
+    cddb_set_timeout(conn, opts.cddb_timeout);
+
   cddb_set_server_port(conn, opts.cddb_port);
+
   if (opts.cddb_http) 
     cddb_http_enable(conn);
   else 
     cddb_http_disable(conn);
+
+  if (NULL != opts.cddb_cachedir) 
+    cddb_cache_set_dir(conn, opts.cddb_cachedir);
+    
+  if (opts.cddb_disable_cache) 
+    cddb_cache_disable(conn);
     
   disc = cddb_disc_new();
   if (!disc) {
@@ -800,7 +866,7 @@ static void
 print_vcd_info(void) {
   vcdinfo_open_return_t open_rc;
   vcdinfo_obj_t obj;
-  open_rc = vcdinfo_open(&obj, &source_name, VCDINFO_SOURCE_AUTO, NULL);
+  open_rc = vcdinfo_open(&obj, &source_name, DRIVER_UNKNOWN, NULL);
   switch (open_rc) {
   case VCDINFO_OPEN_VCD: 
     if (vcdinfo_get_format_version (&obj) == VCD_TYPE_INVALID) {
@@ -935,56 +1001,92 @@ main(int argc, const char *argv[])
   int fs=0;
   poptContext optCon = poptGetContext (NULL, argc, argv, optionsTable, 0);
 
-  gl_default_log_handler = cdio_log_set_handler (_log_handler);
+  gl_default_cdio_log_handler = cdio_log_set_handler (_log_handler);
+#ifdef HAVE_CDDB
+  gl_default_cddb_log_handler = cddb_log_set_handler (_log_handler);
+#endif
 
   program_name = strrchr(argv[0],'/');
   program_name = program_name ? program_name+1 : strdup(argv[0]);
 
   /* Default option values. */
-  opts.silent       = false;
-  opts.debug_level  = 0;
-  opts.no_tracks    = 0;
+  opts.silent        = false;
+  opts.no_header     = false;
+  opts.debug_level   = 0;
+  opts.no_tracks     = 0;
 #ifdef HAVE_CDDB
-  opts.no_cddb      = 0;
-  opts.cddb_port    = 8880;
-  opts.cddb_http    = 0;
+  opts.no_cddb       = 0;
+  opts.cddb_port     = 8880;
+  opts.cddb_http     = 0;
+  opts.cddb_cachedir = NULL;
+  opts.cddb_server   = NULL;
+  opts.cddb_timeout = -1;
+  opts.cddb_disable_cache = false;
+  
 #endif
-  opts.no_ioctl     = 0;
-  opts.no_analysis  = 0;
-  opts.source_image = IMAGE_UNKNOWN;
+  opts.no_ioctl      = 0;
+  opts.no_analysis   = 0;
+  opts.source_image  = IMAGE_UNKNOWN;
      
 
   /* Parse our arguments; every option seen by `parse_opt' will
      be reflected in `arguments'. */
   parse_options(optCon);
      
-  print_version();
+  print_version(opts.version_only);
 
   switch (opts.source_image) {
   case IMAGE_UNKNOWN:
   case IMAGE_AUTO:
     cdio = cdio_open (source_name, DRIVER_UNKNOWN);
+    if (cdio==NULL) {
+      err_exit("%s: Error in automatically selecting driver with input\n", 
+	       program_name);
+    } 
     break;
   case IMAGE_DEVICE:
     cdio = cdio_open (source_name, DRIVER_DEVICE);
+    if (cdio==NULL) {
+      err_exit("%s: Error in automatically selecting device with input\n", 
+	       program_name);
+    } 
     break;
   case IMAGE_BIN:
     cdio = cdio_open (source_name, DRIVER_BINCUE);
+    if (cdio==NULL) {
+      err_exit("%s: Error in opeing bin/cue\n", 
+	       program_name);
+    } 
     break;
   case IMAGE_CUE:
     cdio = cdio_open_cue(source_name);
+    if (cdio==NULL) {
+      err_exit("%s: Error in opening cue/bin with input\n", 
+	       program_name);
+    } 
     break;
   case IMAGE_NRG:
     cdio = cdio_open (source_name, DRIVER_NRG);
+    if (cdio==NULL) {
+      err_exit("%s: Error in opening NRG with input\n", 
+	       program_name);
+    } 
     break;
   }
 
   if (cdio==NULL) {
-    err_exit("%s: Error in finding a usable device driver\n", program_name);
+    err_exit("%s: Error in opening device driver\n", program_name);
   } 
 
+  if (opts.debug_level > 0) {
+    printf("CD driver name: %s\n", cdio_get_driver_name(cdio));
+  }
+  
   if (source_name==NULL) {
     source_name=strdup(cdio_get_arg(cdio, "source"));
+    if (NULL == source_name) {
+      err_exit("%s: No input device given/found\n", program_name);
+    }
   } 
 
   first_track_num = cdio_get_first_track_num(cdio);
