@@ -1,7 +1,7 @@
 #!/bin/sh
 #$Id: check_nrg.sh.in,v 1.2 2003/05/16 21:11:18 rocky Exp $
 
-if test -n "" ; then
+if test -n "-lvcdinfo -lvcd" ; then
   vcd_opt='--no-vcd'
 fi
 
